/*
 * @copyright   © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages/productcomparison/ui/productcompare
 */

/*jsling browser: true, nomen: true*/
/*global define, window*/

define("de_epages/productcomparison/ui/productcompare", [
    "jquery",
    "de_epages",
    "util/storage",

    "jquery/ui/widget",
    "ep/ui/input",
    "ep/ui/tooltip"
], function ($, de_epages, storage) {

    'use strict';

    var warn = (window.console && window.console.warn) || function () {
            return;
        };

    $.widget('ui.productcomparisonUiProductcompare', {

        options: {
            maxItemNo:              8,
            minItemNo:              2,
            productID:              undefined,
            tooltipTooManyProducts: undefined,
            tooltipNoProducts:      undefined
        },

        _create: function () {
            var self = this,
                elem = self.element,
                checkboxID = elem.attr('id'),
                productID = self.options.productID,
                label = $('#' + checkboxID).nextAll('label'),
                link = label.children('a').first();

            /* check product id */
            if (!self._check(productID, link)) {
                return;
            }

            self.options.label = self.label = label;
            self.options.checkboxID = checkboxID;

            /* add 'click' event handler on associated 'Compare' link */
            label.on('click', function (event) {
                self._handleClick(event, link, elem);
            });

            /* create checkbox, (un)tick checkbox, and add 'change' event handler */
            elem.uiInput();
            self._tick(productID);
            elem.on('change', function (event) {
                self._handleChange(event, elem);
            });
        },

        _check: function (productID, link) {
            var self = this;

            if (productID === undefined || isNaN(parseInt(productID, 10)) || link.length === 0) {

                warn('ProductComparison: parameter \'productID\' and/or \'compare\' link missing; product comparison is removed from SF.');

                self.element.remove();

                if (self.label.length > 0) {
                    self.label.remove();
                }

                return false;
            }

            return true;
        },

        _tick: function (productID) {
            var productList = storage.sessionStorage('Product::ProductComparison') || [],
                isSelected = ($.inArray(productID, productList) > -1) ? true : false;
            this.element.prop('checked', isSelected);
        },

        _handleClick: function (event, target, parent) {
            event.preventDefault();

            var productIDs = storage.sessionStorage('Product::ProductComparison') || [],
                url = target.attr('href');

            this._addProductToComparisonList(this.options.productID, productIDs);

            storage.sessionStorage('Product::ProductComparison', productIDs);

            if (productIDs.length < this.options.minItemNo) {
                parent.prop('checked', true);
                this._showTooltip('NoProductsInComparisonList', this.options.tooltipNoProducts, target);
                return;
            }

            $.each(productIDs, function (index, value) {
                url += '&SelectedObjectID=' + value;
            });

            window.location = url;
        },

        _handleChange: function (event, target) {
            var productIDs = storage.sessionStorage('Product::ProductComparison') || [],
                productID = this.options.productID,
                isSelected = false;

            target = $('#' + target.attr('id'));
            if (target.is(':checked')) {
                // add product to comparison list
                if (productIDs.length >= this.options.maxItemNo) {
                    target.prop('checked', isSelected);
                    this._showTooltip('TooManyComparisonProducts', this.options.tooltipTooManyProducts, target);

                    return;
                }

                this._addProductToComparisonList(productID, productIDs);
                isSelected = true;
            } else {
                // remove product from comparison list
                productIDs = $.grep(productIDs, function (value) {
                    return value !== productID;
                });
            }
            target.prop('checked', isSelected);
            storage.sessionStorage('Product::ProductComparison', productIDs);
        },

        _addProductToComparisonList: function (productID, productIDs) {
            var index = $.inArray(productID, productIDs);
            if (index > -1) {  // deletes former occurrence
                productIDs.splice(index, 1);
            }
            productIDs.push(productID);
            while (productIDs.length > this.options.maxItemNo) {
                productIDs.shift();
            }
        },

        _showTooltip: function (id, message, element) {
            if (message === undefined) {
                warn('ProductComparison: No tooltip message is defined.');

                return;
            }

            var template = $($('#' + id + 'Tooltip'));

            if (!template[0]) {
                template = $('<div id="' + id + 'Tooltip" class="Message Medium Notification Wide"><div class="Headline"></div><div class="MessageDetails"><span class="ExtractedContentAreaElement">' + message + '</span></div></div>')
                    .uiTooltip({
                        interactive: true,
                        orientation: 'bottom center',
                        event: 'click'
                    });
            }

            template.uiTooltip({ context: element });
            template.uiTooltip('show');
        },

        destroy: function () {
            this.element.uiInput('destroy');
            $('#' + this.options.checkboxID).off('change');
            this.label.off('click');

            return this._superApply(arguments);
        }

    });

    return de_epages;

});
