/*global define, jQuery, top*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/previeweditor-test',
    'jquery',
    'de_epages/presentation/ui/previeweditor',

    '$ready!'
], function (QUnit, fixture, $, de_epages) {

    QUnit.module("de_epages/presentation/ui/previeweditor", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("de_epages/presentation/ui/previeweditor", function () {
        top.jq = jq = $;
        top.jQuery = top.$ = jQuery;

        // Check prototype methods.
        $.each(['_create', '_setOption', 'destroy', '_execEpAlmostFullscreen', '_afterSetData'], function (i, fn) {

            QUnit.equal(
                $.isFunction($.ui.presentationUiPrevieweditor.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        var $headerDiv = $('#EmailHeader').presentationUiPrevieweditor();
        var $signatureDiv = $('#EmailSignature').presentationUiPrevieweditor();
        var $headerTextarea = $('textarea[name="Header"]');
        var $signatureTextarea = $('textarea[name="Signature"]');


        // Check for *click* handler.
        QUnit.equal(
            $._data($headerDiv[0], 'events') && $._data($headerDiv[0], 'events').click && ($._data($headerDiv[0], 'events').click.length === 1),
            true,
            '*$headerDiv* has *click* handler.'
        );

        QUnit.equal(
            $._data($signatureDiv[0], 'events') && $._data($signatureDiv[0], 'events').click && ($._data($signatureDiv[0], 'events').click.length === 1),
            true,
            '*$signatureDiv* has *click* handler.'
        );

        var headerInstance = $($headerDiv).data('uiPresentationUiPrevieweditor');
        var signatureInstance = $($signatureDiv).data('uiPresentationUiPrevieweditor');

        QUnit.equal(
            signatureInstance.ckeditor.getData(),
            $signatureTextarea.val(),
            'signatureInstance.ckeditor.getData()'
        );

        setTimeout(function () {

            // Open *almostFullscreen*.
            $headerDiv.children().first().trigger('click');

            setTimeout(function () {
                // Check data of *ckeditor* instances.
                QUnit.equal(
                    headerInstance.ckeditor.getData(),
                    $headerTextarea.val(),
                    'headerInstance.ckeditor.getData()'
                );


                // *SetData*. (Closing AlmostFullscreen would trigger this too.)
                headerInstance.ckeditor.setData('AfterSetData method works.');

                setTimeout(function () {
                    QUnit.equal(
                        headerInstance.ckeditor.getData(),
                        $headerTextarea.val(),
                        'headerInstance.ckeditor.getData()'
                    );

                    QUnit.equal(
                        $headerTextarea.val(),
                        'AfterSetData method works.',
                        'headerInstance.ckeditor.getData()'
                    );

                    QUnit.equal(
                        $headerDiv.get(0).innerHTML,
                        'AfterSetData method works.',
                        '$headerDiv.get(0).innerHTML'
                    );

                    // Destroy.
                    headerInstance.destroy();
                    signatureInstance.destroy();

                    QUnit.equal(
                        $._data($headerDiv[0], 'events') && $._data($headerDiv[0], 'events').click,
                        undefined,
                        '*$headerDiv* has *click* handler.'
                    );
                    QUnit.equal(
                        $._data($signatureDiv[0], 'events') && $._data($signatureDiv[0], 'events').click,
                        undefined,
                        '*$signatureDiv* has *click* handler.'
                    );

                    // Check data of *ckeditor* instances.
                    QUnit.deepEqual(
                        $($headerDiv).data('de_epagesPresentationUiPrevieweditor'),
                        $.parseJSON($headerDiv.attr('data-de_epages-presentation-ui-previeweditor') || "null"),
                        "$($headerDiv).data('de_epagesPresentationUiPrevieweditor')"
                    );

                    QUnit.deepEqual(
                        $($signatureDiv).data('de_epagesPresentationUiPrevieweditor'),
                        $.parseJSON($signatureDiv.attr('data-de_epages-presentation-ui-previeweditor') || "null"),
                        "$($signatureDiv).data('de_epagesPresentationUiPrevieweditor')"
                    );

                    // Let's go, QUnit!
                    QUnit.start();
                }, 1000);
            }, 1000);
        }, 5000);
    });
});